#ifndef _MAIN_H
#define _MAIN_H

#include <18F97J60.h>
#device adc=8
	
#FUSES NOWDT                    //No Watch Dog Timer
#FUSES WDT128                   //Watch Dog Timer uses 1:128 Postscale
#FUSES HS                       //High speed Osc (> 4mhz for PCM/PCH) (>10mhz for PCD)
#FUSES NODEBUG                  //No Debug mode for ICD
#FUSES NOXINST                  //Extended set extension and Indexed Addressing mode disabled (Legacy mode)
#FUSES STVREN                   //Stack full/underflow will cause reset
#FUSES NOPROTECT                //Code not protected from reading
#FUSES FCMEN                    //Fail-safe clock monitor enabled
#FUSES IESO                     //Internal External Switch Over mode enabled
#FUSES PRIMARY                  //Primary clock is system clock when scs=00
#FUSES ECCPE                    //Enhanced CCP PWM outpts multiplexed with R6 thorugh RE3
#FUSES NOETHLED              	//Ethernet LED disabled
#FUSES NOWAIT                   //Wait selections unavailable for Table Reads or Table Writes
#FUSES BW16                     //16-bit external bus mode
#FUSES MCU                      //Microcontroller Mode
#FUSES EASHFT                   //Address shifting enabled
                   
#use delay(clock=25000000)
	



/**************** Configuracin puerto serie *********************/

#use rs232(baud=9600,parity=N,xmit=PIN_C6,rcv=PIN_C7,bits=8,stream=PC)

/******** Constantes para acceso a las puertas del PIC ********/

#byte PORTA = 0xf80	// PORTA en f80h
#byte PORTB = 0xf81	// PORTB en f81h
#byte PORTC = 0xf82	// PORTC en f82h
#byte PORTD = 0xf83	// PORTD en f83h
#byte PORTE = 0xf84	// PORTE en f84h
#byte PORTF = 0xf85	// PORTF en f85h
#byte PORTG = 0xf86	// PORTG en f86h
#byte PORTH = 0xf87	// PORTH en f87h
#byte PORTJ = 0xf88	// PORTJ en f88h


/*********** Se definen los tipos de modos y tipos de trama ******/

#define ECHO 				0
#define ACK_ECHO 			1
#define READ_IN 			2
#define SEND_IN 			3
#define WRITE_IN 			4
#define ACK_WRITE 			5
#define NO_ACK_WRITE 		6
#define STAND_BYE 			7
#define POLLING				8
#define EVENT 				9
#define STOP 				10 

#define WRITE_IN_ONE		11
#define ACK_WRITE_ONE		12
#define NO_ACK_WRITE_ONE 	13

#define PortSerie 			20
#define PortEthernet 		30


/*******************  Longitud de la trama  ****************/

#define LONG_TRAMA 14				// Longitud de trama
#define NUM_PUERTAS 64  			// Establecer siempre valores multiplos de 8. Ej 8 = 1 puerta, 16 = 2 puertas

/********** Variables utilizadas para el muestreo ***********/

int8 s_old_in[64];					// Array de byte utilizado para almacenar el estado actual de las puertas
int8 s_new_in[64]; 					// Array de byte utilizado para almacenar el estado de las puertas al escanear
int1 s_temp[64];					// Array de bits utilizado temporalmente para enviar la trama al PC
int8 c_in[NUM_PUERTAS];				// Array para almacenar los cambios de flancos de las puertas
int8 th_in;							// Variable utilizada para identificar el umbral de cambios maximos
int8 cont_red=0;					/* Variable utilizada necesaria para contar los desbordamientos del 
									 * TIMER0 en modo red, ya que este desborda cada 100ms, con lo que sumamos
									 * 1 cada segundo */
int8 cont_serie=0;					/* Variable utilizada para contar los desbordamientos del TIMER0 en modo
									 * serie, sumamos 1 cada medio segundo */
						

/* ************************************ */

/*********************  ID del PIC  ***************************/

int8 ID_PIC=8;						// Id del PIC por defecto

/* Variables necesarias para la trama */

unsigned char tx_trama[LONG_TRAMA], rx_trama[LONG_TRAMA];
unsigned int contador[2];                                       
                                          
/* ************************************ */    

int8 STATUS=STAND_BYE; 				// Variable de Control de los estados del PIC
int8 count_keepalive=0;				// Contador usado para el keep alive
int8 numKeepAlive;					// Valor recibio de keep alive numero de X segundos

/******** Variables Booleanas ******/

int1 flag_red=FALSE;				// Bandera de seleccion de modo red
int1 flag_serie=FALSE; 				// Bandera de seleccion de modo serie
int1 enable_listen=FALSE;			// Utilizada en modo red, para la escucha de paquetes UDP
int1 recibo=FALSE;     				// Si se ha recibido o no datos por el RS232
int1 wait=TRUE;     				// Utilizada para la espera de recepcin de datos
int1 keepalive=FALSE;    			// Estado del keepalive
int1 proceso=TRUE;     				// Saber si se llama o no a la funcion proceso_Rx
int1 toggle=FALSE; 					// Para saber si ha habido cambio o no en alguna de las puertas, superando el umbral

/**************************************/

// Utilizado para encendido, apagado de una puerta

#define ON output_high
#define OFF output_low

void status_proceso();
void status_recibo();
void status_toggle();
void status_muestreo();


#endif
                 
                    
                      

                   

